﻿' add System.DirectoryServices to References
Imports System.DirectoryServices
Imports System.Data.OleDb
Imports System.Net.Mail

Module Module1
    Dim Full_Name, Body As String
    Dim dt_history As New DataTable
    Dim lock_cnt As Integer
    Dim HasChanges As Boolean

    Sub Main()
        Dim con As New OleDbConnection
        Dim cmd As New OleDbCommand
        Dim cb As OleDbCommandBuilder
        Dim da As New OleDbDataAdapter

        Dim dir As DirectoryEntry
        Dim el As New EventLog

        Console.WriteLine("LockoutAlert")
        Console.WriteLine("Start: " & Now())

        ' fill the data tables
        Console.WriteLine("Reading the table")
        Try
            con.ConnectionString = My.Settings.ConnectString
            cmd.CommandText = "Select * from History"
            cmd.Connection = con
            da.SelectCommand = cmd
            da.Fill(dt_history)
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            Console.WriteLine("Yikes! Can't read the table" & vbCr & ex.Message)
            el.Source = "LockoutAlert"
            el.WriteEntry("Yikes! Can't read the table" & vbCr & ex.Message, EventLogEntryType.Error)
            Exit Sub
        End Try
        ' Generate the list of emails
        If My.Settings.Email_List.Count <= 0 Then
            Console.WriteLine("Yikes! No email addresses defined!")
            el.Source = "LockoutAlert"
            el.WriteEntry("Yikes! No email addresses defined!", EventLogEntryType.Warning)
            ' not fatal
        End If

        ' Find our users
        Body = ""
        lock_cnt = 0
        HasChanges = False
        Console.WriteLine("Getting info on Domain Users")
        Try
            Dim srch As DirectorySearcher
            Dim result As SearchResult
            Dim de As DirectoryEntry

            Full_Name = "displayName"

            de = New DirectoryEntry("LDAP://" & My.Settings.Domain & "/" & My.Settings.OU)
            srch = New DirectoryServices.DirectorySearcher(de)
            srch.Filter = "(&(objectClass=user)(objectCategory=person))"
            For Each result In srch.FindAll()
                dir = result.GetDirectoryEntry
                DoUsers(dir)
            Next
        Catch ex As Exception
            Environment.ExitCode = Err.Number
            Console.WriteLine("Internal Error" & vbCr & ex.Message & vbCr & ex.StackTrace)
            el.Source = "LockoutAlert"
            el.WriteEntry("Internal Error" & vbCr & ex.Message & vbCr & ex.StackTrace, EventLogEntryType.Error)
            Exit Sub
        End Try
        Console.WriteLine()

        ' Anything to report?
        If Body <> "" Then
            Console.WriteLine("Sending emails")
            SendEmail(Body, "Accounts Locked")
        End If

        ' Save the history data
        If HasChanges Then
            Console.WriteLine("Saving data")
            Try
                cb = New OleDbCommandBuilder(da)
                da.Update(dt_history)
            Catch ex As Exception
                Environment.ExitCode = Err.Number
                Console.WriteLine("Yikes! Can't update the table" & vbCr & ex.Message)
                el.Source = "LockoutAlert"
                el.WriteEntry("Yikes! Can't update the table" & vbCr & ex.Message, EventLogEntryType.Error)
                Exit Sub
            End Try
        End If

        ' clean up and go home
        el.Source = "LockoutAlert"
        el.WriteEntry(lock_cnt & " accounts locked" & vbCrLf & Body, EventLogEntryType.Information)

        Console.WriteLine("End: " & Now())
        System.Threading.Thread.Sleep(5000)
    End Sub

    '
    ' Get the fullname, check to see if they are locked
    '
    Sub DoUsers(ByVal dir As DirectoryEntry)
        Dim Name, FullName, Description As String
        Dim i As Integer
        Dim Acct_Locked, isDup As Boolean
        Dim dr As DataRow

        Name = dir.Name.Replace("CN=", "")
        FullName = dir.Properties(Full_Name).Value.ToString
        Description = ""
        If Not IsNothing(dir.Properties("Description").Value) Then
            Description = dir.Properties("Description").Value.ToString
        End If

        ' Is the account locked?
        i = CInt(dir.Properties("UserFlags").Value)
        Acct_Locked = CBool(i And 16)
        If Acct_Locked And Not isDup Then
            Console.Write("!")
            ' Let's see if this is a new entry or not
            isDup = False
            For Each dr In dt_history.Rows
                If dr("Name").ToString = Name AndAlso DateDiff(DateInterval.Hour, CDate(dr("CurrDate")), Now) <= My.Settings.Frequency + 1 Then
                    dr("CurrDate") = Now
                    isDup = True
                    HasChanges = True
                    Exit For
                End If
            Next
            If Not isDup Then
                ' record this in the history table
                dr = dt_history.NewRow
                dr("Name") = Name
                dr("FullName") = FullName
                dr("CurrDate") = Now
                dt_history.Rows.Add(dr)
                HasChanges = True
            End If
            lock_cnt += 1
            ' save in the email body
            Body = Body & "User '" & Name & "' ('" & FullName & "', " & Description & ") is locked out" & vbCrLf
        Else
            Console.Write(".")
        End If
    End Sub

    '
    ' Send an email to a list of users
    '
    Sub SendEmail(ByVal buf As String, ByVal subject As String)
        Dim mailClient As SmtpClient
        Dim Email As String

        mailClient = New SmtpClient(My.Settings.SMTP_Server)

        For Each Email In My.Settings.Email_List
            ' must have a valid From address to get past our spam filters
            Dim Message As New MailMessage(My.Settings.Sender, Email)

            Message.Subject = subject
            Message.Body = buf

            mailClient.Send(Message)
        Next

    End Sub
End Module
